<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use Doctrine\ORM\EntityManager;

class Inteclie extends RepositoryBase {

    Use TQuery;

    protected $table = 'INTECLIE';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    public function updateIdativoInteclie($cdoperultatu, $nrorg, $nrparcnegocio)
    {
        $query = "UPDATE INTECLIE
                     SET IDATIVO        = :IDATIVO,
                         CDOPERULTATU   = :CDOPERULTATU,
                         NRORGULTATU    = :NRORGULTATU,
                         DTULTATU       = SYSDATE
                   WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                               FROM RELACIONAPARC
                                              WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                                AND NRORG           = :NRORG
                                                AND NRTIPORELACIONA = :NRTIPORELACIONA)
                     AND NRORG = :NRORG";

        $params = array(
            "NRORG"           => $nrorg,
            "NRPARCNEGOCIO"   => $nrparcnegocio,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $cdoperultatu,
            "IDATIVO"         => "N",
            "NRTIPORELACIONA" => 25
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function activateInteclie($nrorg, $nrparcnegocio, $cdcliente, $cdoperultatu, $idativo)
    {
        $query =  "UPDATE INTECLIE
                      SET IDATIVO         = :IDATIVO,
                          CDOPERULTATU    = :CDOPERULTATU,
                          NRORGULTATU     = :NRORGULTATU,
                          DTULTATU        = SYSDATE
                    WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                                FROM RELACIONAPARC
                                               WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                              AND NRORG           = :NRORG
                                              AND NRTIPORELACIONA = :NRTIPORELACIONA)
                      AND NRORG           = :NRORG
                      AND CDCLIENTE       = :CDCLIENTE";

        $params = array(
            "NRORG"           => $nrorg,
            "NRPARCNEGOCIO"   => $nrparcnegocio,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $cdoperultatu,
            "IDATIVO"         => $idativo,
            "NRTIPORELACIONA" => 25,
            "CDCLIENTE"       => $cdcliente
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
}
