import { Http, IEventParam, Loader, Metadata, Singleton } from '@zeedhi/core';
import { OperadorController } from '@/controllers/Operador/OperadorController';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { PmLibAlertService, PmLibDialogService, PmLibFormSave, PmLibHttp, PmLibUtil } from '@zeedhi/pm-lib';
import { Utilities } from '@/Util/Utilities';
import { Button, Form, IGridEvent, LoadingService, Modal, ModalService, Password, Select, TextInput } from '@zeedhi/common';
import AclMessages from '@/Util/AclMessages';
import { OperadorService } from '@/services/OperadorService';

@Singleton
export class OperadorGridController {
	public operadorControler: OperadorController;

	public operadorService: OperadorService;

	public readonlyNrorgoperFieldFilter = false;

	public operadorForChangePassword: any = {};

	public get isVisibleOperadorGrid() {
		return this.operadorControler.isVisibleGridAndDetail;
	}

	get visibleCdfilialFieldFilter() {
		return this.operadorControler.showFullOperatorWindow &&
		this.operadorControler.filialRuleInFullOperatorWindow &&
		this.operadorControler.operador.ACL_ADMINISTRATOR;
	}

	public get visibleOnlyInCompleteMode(): boolean {
		return this.operadorControler.showFullOperatorWindow;
	}

	constructor() {
		this.operadorControler = Loader.getInstance('OperadorController');
		this.operadorService = new OperadorService();
	}

	public getInstanceOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('operadorGrid');
	}

	/**
	 * Event methods
	 */

	public async onMountedOperadorGrid() {
		const grid = this.getInstanceOperadorGrid();
		const operadorInfo = await this.operadorControler.promiseOperadorInfo;
		const orgoper = await this.operadorControler.promiseOrgoper;

		ModalService.createFromJson(this.addOperadorModalPath, true).then((response) => {
			this.addOperadorModal = response;
		});

		ModalService.createFromJson(this.changePasswordOperadorModalPath, true).then((response) => {
			this.changePasswordOperadorModal = response;
		});

		this.setOperadorGridDynamicFilter(grid, operadorInfo, orgoper);
		this.defineConfigOnMountedOperadorGrid(orgoper);

		if (
			this.operadorControler.showFullOperatorWindow &&
			this.operadorControler.filialRuleInFullOperatorWindow &&
			!operadorInfo.ACL_ADMINISTRATOR &&
			operadorInfo.CDFILIAL.length
		) {
			grid.datasource.addFilter('CDFILIAL', operadorInfo.CDFILIAL);
		}

		try {
			await grid.datasource.get();
		} catch (e) {
			PmLibDialogService.showErrorBackend(e);
		}
	}

	private setOperadorGridDynamicFilter(grid: TekGrid, operadirInfo: any, orgoper: any) {
		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				IDATIVO: [{
					operation: 'EQUALS',
					relation: 'AND',
					value: 'S',
				}],
			};

			if (orgoper.length === 1) {
				grid.datasource.dynamicFilter.NRORGOPER = [{
					operation: 'IN',
					relation: 'AND',
					value: [orgoper[0].NRORG],
				}];
			}
			Utilities.dynamicFilterLog('Operador', grid.datasource.dynamicFilter);
		}
	}

	private defineConfigOnMountedOperadorGrid(orgoper: any) {
		this.readonlyNrorgoperFieldFilter = orgoper.length === 1;
	}

	public onBeforeDestroyOperadorGrid() {
		this.addOperadorModal?.destroy();
		this.changePasswordOperadorModal?.destroy();
	}

	public addClickOperadorGrid() {
		this.addOperadorModal.show();
	}

	public async rowClickOperadorGrid({ row }: IGridEvent) {
		this.operadorControler.currentOperador = row;
		this.operadorControler.isVisibleGridAndDetail = false;
		if (!row.NRPARCNEGOCIO) {
			await this.operadorService.createParcnegocio(row);
		}
		this.setOperadorFormEdit(row);
		this.operadorControler.backupRow = PmLibUtil.clone(row);
		this.changeOperadorDetailTabs();
	}

	private setOperadorFormEdit(row: object) {
		const operadorFormEdit = Metadata.getInstance<PmLibFormSave>('operadorFormEdit');
		operadorFormEdit.value = row;
	}

	public async changeOperadorDetailTabs() {
		this.operadorControler.changeOperadorDetailTabs();
	}

	public clickChangePasswordActionButton({ event, row }: IEventParam<Button>) {
		event?.stopPropagation();
		this.operadorForChangePassword = row;
		this.changePasswordOperadorModal.show();
	}

	/** Add Operador Modal */

	private addOperadorModal!: Modal;

	public addOperadorModalPath: string = 'components/operador/grid/modais/addOperador';

	public async onMountedAddOperadorFormModal() {
		const form = Metadata.getInstance<Form>('addOperadorFormModal');
		const cdoperadorField = form.getChildInstance<TextInput>('CDOPERADOR');
		const cpfField = form.getChildInstance<TextInput>('NRINSCRICAOPARC');
		const cdpaisField = form.getChildInstance<Select>('CDPAIS');
		const nmorganizacaooperField = form.getChildInstance<Select>('NRORGOPER');
		const nrorgtrabField = form.getChildInstance<TextInput>('NRORGTRAB');
		const idopermasterField = form.getChildInstance<TextInput>('IDOPERMASTER');
		const cdfilialField = form.getChildInstance<Select>('CDFILIAL');

		try {
			LoadingService.show();
			// const { data: { data } } = await Http.get('/paramgeral');
			/**
			 * Definição de valores e congigurações dos fields.
			 * Ex.: se vai ser readonly, visível, obrigatório ou não.
			 */
			const paramgerlPromise = Http.get('/paramgeral').then(({ data: { data } }) => {
				this.operadorService.setConfigCdoperadorAddField(data, cdoperadorField);
				this.operadorService.setConfigCpfAddField(data, cpfField);
			});

			idopermasterField.readonly = this.operadorControler.operador.IDOPERMASTER !== 'S';
			nrorgtrabField.value = this.operadorControler.operador.NMORGTRAB;
			const confignmorgoperPromise = this.operadorService.setConfigNmorganizacaooperAddField(nmorganizacaooperField, this.operadorControler.nrorg);
			const configCdfilialPromise = this.operadorService.setConfigCdfilialAddField(cdfilialField,
																					this.operadorControler.showFullOperatorWindow,
																					this.operadorControler.operador.CDFILIAL);
			const cdPaisPromise = cdpaisField.setValue('0055');

			await Promise.all([confignmorgoperPromise, configCdfilialPromise, cdPaisPromise, paramgerlPromise]);

			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public changeFieldCdoperador({ component }: IEventParam<TextInput>) {
		const form = Metadata.getInstance<Form>('addOperadorFormModal');
		// eslint-disable-next-line no-restricted-globals
		if (!isNaN(component.value)) {
			form.value.CDOPERADOR = PmLibUtil.fill(component.value, 12, false, false);
		}
	}

	public closeAddOperadorModal() {
		this.addOperadorModal.hide();
	}

	public async saveAddOperadorModal() {
		const addForm = Metadata.getInstance<PmLibFormSave>('addOperadorFormModal');
		const operadorGrid = this.getInstanceOperadorGrid();

		if (addForm.validate()) {
			addForm.value.NRINSCRICAOPARC = PmLibUtil.getOnlyNumberInString(addForm.value.NRINSCRICAOPARC);
			addForm.value.NRPERFILACESSO = localStorage.getItem('CDPERFILSELECIONADO');
			addForm.value.CDTIPOINSCRICAO = this.operadorService.getInscriptionType(addForm.value.NRINSCRICAOPARC);
			addForm.value.SHOW_FULL_OPERATOR = (window.showFullOperatorWindow || false);
			addForm.value.IDOPERATIVO = (addForm.value.IDATIVO === 'S') ? 'A' : 'I';
			addForm.value.ADMINISTRATOR = false;

			await PmLibHttp.$post('/operador', addForm.value);

			this.closeAddOperadorModal();
			operadorGrid.reload();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}

	/** Change password Operador Modal */

	private changePasswordOperadorModal!: Modal;

	public changePasswordOperadorModalPath: string = 'components/operador/grid/modais/changePassword';

	public get getOperadorName(): string {
		return this.operadorForChangePassword.NMOPERADOR || '';
	}

	public closeChangePasswordOperadorModal() {
		this.changePasswordOperadorModal.hide();
	}

	public async saveChangePasswordOperadorModal() {
		const form = Metadata.getInstance<PmLibFormSave>('changePasswordOperadorFormModal');
		const passwordField = form.getChildInstance<Password>('PASSWORD');
		const passwordConfirmField = form.getChildInstance<Password>('PASSWORD_CONFIRM');
		const dataset = {
			PASSWORD: passwordField.value,
			PASSWORD_CONFIRM: passwordConfirmField.value,
			CDOPERADOR: this.operadorForChangePassword.CDOPERADOR,
			DSEMAILOPER: this.operadorForChangePassword.DSEMAILOPER,
		};

		await PmLibHttp.$post('/reset-password', dataset);
		this.changePasswordOperadorModal.hide();
		PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
	}
}
