import { Metadata, Validation } from '@zeedhi/core';
import { Form, TextInput } from '@zeedhi/common';
import { PmLibValidations } from '@zeedhi/pm-lib';

const cdoperadorLengthValidations = () => {
	const message = 'Código deve conter 12 caracteres.';
	return (value: string) => (value && value.length === 12) || message;
};

const cpfValidation = () => {
	const message = 'CPF inválido';
	return (value: string) => (PmLibValidations.validateCpf(value) || (value === null || !value.length)) || message;
};

const confirmPasswordValidation = (config: any) => {
	const addOperadorForm = Metadata.getInstance<Form>(config.form);
	const passwordField = addOperadorForm.getChildInstance<TextInput>(config.field);
	const message = 'Senhas não conferem.';
	return (value: string) => value === passwordField.value || message;
};

Validation.register('cdoperadorLength', cdoperadorLengthValidations);
Validation.register('cpf', cpfValidation);
Validation.register('confirmPassword', confirmPasswordValidation);
